/*************** from legacy jquery.carousel file ***************/

/*!
 * jCarousel - Riding carousels with jQuery
 *   http://sorgalla.com/jcarousel/
 *
 * Copyright (c) 2006 Jan Sorgalla (http://sorgalla.com)
 * Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php)
 * and GPL (http://www.opensource.org/licenses/gpl-license.php) licenses.
 *
 * Built on top of the jQuery library
 *   http://jquery.com
 *
 * Inspired by the "Carousel Component" by Bill Scott
 *   http://billwscott.com/carousel/
 */

/*global window, jQuery */
(function ($) {
    // Default configuration properties.
    var defaults = {
        vertical: false,
        rtl: false,
        start: 1,
        offset: 1,
        size: null,
        scroll: 3,
        visible: null,
        animation: 'normal',
        easing: 'swing',
        auto: 0,
        wrap: null,
        initCallback: null,
        setupCallback: null,
        reloadCallback: null,
        itemLoadCallback: null,
        itemFirstInCallback: null,
        itemFirstOutCallback: null,
        itemLastInCallback: null,
        itemLastOutCallback: null,
        itemVisibleInCallback: null,
        itemVisibleOutCallback: null,
        animationStepCallback: null,
        buttonNextHTML: '<div></div>',
        buttonPrevHTML: '<div></div>',
        buttonNextEvent: 'click',
        buttonPrevEvent: 'click',
        buttonNextCallback: null,
        buttonPrevCallback: null,
        itemFallbackDimension: null
    }, windowLoaded = false;

    $(window).bind('load.jcarousel', function () { windowLoaded = true; });

    /**
     * The jCarousel object.
     *
     * @constructor
     * @class jcarousel
     * @param e {HTMLElement} The element to create the carousel for.
     * @param o {Object} A set of key/value pairs to set as configuration properties.
     * @cat Plugins/jCarousel
     */
    $.jcarousel = function (e, o) {
        this.options = $.extend({}, defaults, o || {});

        this.locked = false;
        this.autoStopped = false;

        this.container = null;
        this.clip = null;
        this.list = null;
        this.buttonNext = null;
        this.buttonPrev = null;
        this.buttonNextState = null;
        this.buttonPrevState = null;

        // Only set if not explicitly passed as option
        if (!o || o.rtl === undefined) {
            this.options.rtl = ($(e).attr('dir') || $('html').attr('dir') || '').toLowerCase() == 'rtl';
        }

        this.wh = !this.options.vertical ? 'width' : 'height';
        this.lt = !this.options.vertical ? (this.options.rtl ? 'right' : 'left') : 'top';

        // Extract skin class
        var skin = '', split = e.className.split(' ');

        for (var i = 0; i < split.length; i++) {
            if (split[i].indexOf('jcarousel-skin') != -1) {
                $(e).removeClass(split[i]);
                skin = split[i];
                break;
            }
        }

        if (e.nodeName.toUpperCase() == 'UL' || e.nodeName.toUpperCase() == 'OL') {
            this.list = $(e);
            this.clip = this.list.parents('.jcarousel-clip');
            this.container = this.list.parents('.jcarousel-container');
        } else {
            this.container = $(e);
            this.list = this.container.find('ul,ol').eq(0);
            this.clip = this.container.find('.jcarousel-clip');
        }

        if (this.clip.size() === 0) {
            this.clip = this.list.wrap('<div></div>').parent();
        }

        if (this.container.size() === 0) {
            this.container = this.clip.wrap('<div></div>').parent();
        }

        if (skin !== '' && this.container.parent()[0].className.indexOf('jcarousel-skin') == -1) {
            this.container.wrap('<div class=" ' + skin + '"></div>');
        }

        this.buttonPrev = $('.jcarousel-prev', this.container);

        if (this.buttonPrev.size() === 0 && this.options.buttonPrevHTML !== null) {
            this.buttonPrev = $(this.options.buttonPrevHTML).appendTo(this.container);
        }

        this.buttonPrev.addClass(this.className('jcarousel-prev'));

        this.buttonNext = $('.jcarousel-next', this.container);

        if (this.buttonNext.size() === 0 && this.options.buttonNextHTML !== null) {
            this.buttonNext = $(this.options.buttonNextHTML).appendTo(this.container);
        }

        this.buttonNext.addClass(this.className('jcarousel-next'));

        this.clip.addClass(this.className('jcarousel-clip')).css({
            position: 'relative'
        });

        this.list.addClass(this.className('jcarousel-list')).css({
            overflow: 'hidden',
            position: 'relative',
            top: 0,
            margin: 0,
            padding: 0
        }).css((this.options.rtl ? 'right' : 'left'), 0);

        this.container.addClass(this.className('jcarousel-container')).css({
            position: 'relative'
        });

        if (!this.options.vertical && this.options.rtl) {
            this.container.addClass('jcarousel-direction-rtl').attr('dir', 'rtl');
        }

        var di = this.options.visible !== null ? Math.ceil(this.clipping() / this.options.visible) : null;
        var li = this.list.children('li');

        var self = this;

        if (li.size() > 0) {
            var wh = 0, j = this.options.offset;
            li.each(function () {
                self.format(this, j++);
                wh += self.dimension(this, di);
            });

            this.list.css(this.wh, (wh + 100) + 'px');

            // Only set if not explicitly passed as option
            if (!o || o.size === undefined) {
                this.options.size = li.size();
            }
        }

        // For whatever reason, .show() does not work in Safari...
        this.container.css('display', 'block');
        this.buttonNext.css('display', 'block');
        this.buttonPrev.css('display', 'block');

        this.funcNext = function () { self.next(); return false; };
        this.funcPrev = function () { self.prev(); return false; };
        this.funcResize = function () {
            if (self.resizeTimer) {
                clearTimeout(self.resizeTimer);
            }

            self.resizeTimer = setTimeout(function () {
                self.reload();
            }, 100);
        };

        if (this.options.initCallback !== null) {
            this.options.initCallback(this, 'init');
        }

        if (!windowLoaded && $jc.isSafari()) {
            this.buttons(false, false);
            $(window).bind('load.jcarousel', function () { self.setup(); });
        } else {
            this.setup();
        }
    };

    // Create shortcut for internal use
    var $jc = $.jcarousel;

    $jc.fn = $jc.prototype = {
        jcarousel: '0.2.9'
    };

    $jc.fn.extend = $jc.extend = $.extend;

    $jc.fn.extend({
        /**
         * Setups the carousel.
         *
         * @method setup
         * @return undefined
         */
        setup: function () {
            this.first = null;
            this.last = null;
            this.prevFirst = null;
            this.prevLast = null;
            this.animating = false;
            this.timer = null;
            this.resizeTimer = null;
            this.tail = null;
            this.inTail = false;

            if (this.locked) {
                return;
            }

            this.list.css(this.lt, this.pos(this.options.offset) + 'px');
            var p = this.pos(this.options.start, true);
            this.prevFirst = this.prevLast = null;
            this.animate(p, false);

            $(window).unbind('resize.jcarousel', this.funcResize).bind('resize.jcarousel', this.funcResize);

            if (this.options.setupCallback !== null) {
                this.options.setupCallback(this);
            }
        },

        /**
         * Clears the list and resets the carousel.
         *
         * @method reset
         * @return undefined
         */
        reset: function () {
            this.list.empty();

            this.list.css(this.lt, '0px');
            this.list.css(this.wh, '10px');

            if (this.options.initCallback !== null) {
                this.options.initCallback(this, 'reset');
            }

            this.setup();
        },

        /**
         * Reloads the carousel and adjusts positions.
         *
         * @method reload
         * @return undefined
         */
        reload: function () {
            if (this.tail !== null && this.inTail) {
                this.list.css(this.lt, $jc.intval(this.list.css(this.lt)) + this.tail);
            }

            this.tail = null;
            this.inTail = false;

            if (this.options.reloadCallback !== null) {
                this.options.reloadCallback(this);
            }

            if (this.options.visible !== null) {
                var self = this;
                var di = Math.ceil(this.clipping() / this.options.visible), wh = 0, lt = 0;
                this.list.children('li').each(function (i) {
                    wh += self.dimension(this, di);
                    if (i + 1 < self.first) {
                        lt = wh;
                    }
                });

                this.list.css(this.wh, wh + 'px');
                this.list.css(this.lt, -lt + 'px');
            }

            this.scroll(this.first, false);
        },

        /**
         * Locks the carousel.
         *
         * @method lock
         * @return undefined
         */
        lock: function () {
            this.locked = true;
            this.buttons();
        },

        /**
         * Unlocks the carousel.
         *
         * @method unlock
         * @return undefined
         */
        unlock: function () {
            this.locked = false;
            this.buttons();
        },

        /**
         * Sets the size of the carousel.
         *
         * @method size
         * @return undefined
         * @param s {Number} The size of the carousel.
         */
        size: function (s) {
            if (s !== undefined) {
                this.options.size = s;
                if (!this.locked) {
                    this.buttons();
                }
            }

            return this.options.size;
        },

        /**
         * Checks whether a list element exists for the given index (or index range).
         *
         * @method get
         * @return bool
         * @param i {Number} The index of the (first) element.
         * @param i2 {Number} The index of the last element.
         */
        has: function (i, i2) {
            if (i2 === undefined || !i2) {
                i2 = i;
            }

            if (this.options.size !== null && i2 > this.options.size) {
                i2 = this.options.size;
            }

            for (var j = i; j <= i2; j++) {
                var e = this.get(j);
                if (!e.length || e.hasClass('jcarousel-item-placeholder')) {
                    return false;
                }
            }

            return true;
        },

        /**
         * Returns a jQuery object with list element for the given index.
         *
         * @method get
         * @return jQuery
         * @param i {Number} The index of the element.
         */
        get: function (i) {
            return $('>.jcarousel-item-' + i, this.list);
        },

        /**
         * Adds an element for the given index to the list.
         * If the element already exists, it updates the inner html.
         * Returns the created element as jQuery object.
         *
         * @method add
         * @return jQuery
         * @param i {Number} The index of the element.
         * @param s {String} The innerHTML of the element.
         */
        add: function (i, s) {
            var e = this.get(i), old = 0, n = $(s);

            if (e.length === 0) {
                var c, j = $jc.intval(i);
                e = this.create(i);
                while (true) {
                    c = this.get(--j);
                    if (j <= 0 || c.length) {
                        if (j <= 0) {
                            this.list.prepend(e);
                        } else {
                            c.after(e);
                        }
                        break;
                    }
                }
            } else {
                old = this.dimension(e);
            }

            if (n.get(0).nodeName.toUpperCase() == 'LI') {
                e.replaceWith(n);
                e = n;
            } else {
                e.empty().append(s);
            }

            this.format(e.removeClass(this.className('jcarousel-item-placeholder')), i);

            var di = this.options.visible !== null ? Math.ceil(this.clipping() / this.options.visible) : null;
            var wh = this.dimension(e, di) - old;

            if (i > 0 && i < this.first) {
                this.list.css(this.lt, $jc.intval(this.list.css(this.lt)) - wh + 'px');
            }

            this.list.css(this.wh, $jc.intval(this.list.css(this.wh)) + wh + 'px');

            return e;
        },

        /**
         * Removes an element for the given index from the list.
         *
         * @method remove
         * @return undefined
         * @param i {Number} The index of the element.
         */
        remove: function (i) {
            var e = this.get(i);

            // Check if item exists and is not currently visible
            if (!e.length || (i >= this.first && i <= this.last)) {
                return;
            }

            var d = this.dimension(e);

            if (i < this.first) {
                this.list.css(this.lt, $jc.intval(this.list.css(this.lt)) + d + 'px');
            }

            e.remove();

            this.list.css(this.wh, $jc.intval(this.list.css(this.wh)) - d + 'px');
        },

        /**
         * Moves the carousel forwards.
         *
         * @method next
         * @return undefined
         */
        next: function () {
            if (this.tail !== null && !this.inTail) {
                this.scrollTail(false);
            } else {
                this.scroll(((this.options.wrap == 'both' || this.options.wrap == 'last') && this.options.size !== null && this.last == this.options.size) ? 1 : this.first + this.options.scroll);
            }
        },

        /**
         * Moves the carousel backwards.
         *
         * @method prev
         * @return undefined
         */
        prev: function () {
            if (this.tail !== null && this.inTail) {
                this.scrollTail(true);
            } else {
                this.scroll(((this.options.wrap == 'both' || this.options.wrap == 'first') && this.options.size !== null && this.first == 1) ? this.options.size : this.first - this.options.scroll);
            }
        },

        /**
         * Scrolls the tail of the carousel.
         *
         * @method scrollTail
         * @return undefined
         * @param b {Boolean} Whether scroll the tail back or forward.
         */
        scrollTail: function (b) {
            if (this.locked || this.animating || !this.tail) {
                return;
            }

            this.pauseAuto();

            var pos = $jc.intval(this.list.css(this.lt));

            pos = !b ? pos - this.tail : pos + this.tail;
            this.inTail = !b;

            // Save for callbacks
            this.prevFirst = this.first;
            this.prevLast = this.last;

            this.animate(pos);
        },

        /**
         * Scrolls the carousel to a certain position.
         *
         * @method scroll
         * @return undefined
         * @param i {Number} The index of the element to scoll to.
         * @param a {Boolean} Flag indicating whether to perform animation.
         */
        scroll: function (i, a) {
            if (this.locked || this.animating) {
                return;
            }

            this.pauseAuto();
            this.animate(this.pos(i), a);
        },

        /**
         * Prepares the carousel and return the position for a certian index.
         *
         * @method pos
         * @return {Number}
         * @param i {Number} The index of the element to scoll to.
         * @param fv {Boolean} Whether to force last item to be visible.
         */
        pos: function (i, fv) {
            var pos = $jc.intval(this.list.css(this.lt));

            if (this.locked || this.animating) {
                return pos;
            }

            if (this.options.wrap != 'circular') {
                i = i < 1 ? 1 : (this.options.size && i > this.options.size ? this.options.size : i);
            }

            var back = this.first > i;

            // Create placeholders, new list width/height
            // and new list position
            var f = this.options.wrap != 'circular' && this.first <= 1 ? 1 : this.first;
            var c = back ? this.get(f) : this.get(this.last);
            var j = back ? f : f - 1;
            var e = null, l = 0, p = false, d = 0, g;

            while (back ? --j >= i : ++j < i) {
                e = this.get(j);
                p = !e.length;
                if (e.length === 0) {
                    e = this.create(j).addClass(this.className('jcarousel-item-placeholder'));
                    c[back ? 'before' : 'after'](e);

                    if (this.first !== null && this.options.wrap == 'circular' && this.options.size !== null && (j <= 0 || j > this.options.size)) {
                        g = this.get(this.index(j));
                        if (g.length) {
                            e = this.add(j, g.clone(true));
                        }
                    }
                }

                c = e;
                d = this.dimension(e);

                if (p) {
                    l += d;
                }

                if (this.first !== null && (this.options.wrap == 'circular' || (j >= 1 && (this.options.size === null || j <= this.options.size)))) {
                    pos = back ? pos + d : pos - d;
                }
            }

            // Calculate visible items
            var clipping = this.clipping(), cache = [], visible = 0, v = 0;
            c = this.get(i - 1);
            j = i;

            while (++visible) {
                e = this.get(j);
                p = !e.length;
                if (e.length === 0) {
                    e = this.create(j).addClass(this.className('jcarousel-item-placeholder'));
                    // This should only happen on a next scroll
                    if (c.length === 0) {
                        this.list.prepend(e);
                    } else {
                        c[back ? 'before' : 'after'](e);
                    }

                    if (this.first !== null && this.options.wrap == 'circular' && this.options.size !== null && (j <= 0 || j > this.options.size)) {
                        g = this.get(this.index(j));
                        if (g.length) {
                            e = this.add(j, g.clone(true));
                        }
                    }
                }

                c = e;
                d = this.dimension(e);
                if (d === 0) {
                    throw new Error('jCarousel: No width/height set for items. This will cause an infinite loop. Aborting...');
                }

                if (this.options.wrap != 'circular' && this.options.size !== null && j > this.options.size) {
                    cache.push(e);
                } else if (p) {
                    l += d;
                }

                v += d;

                if (v >= clipping) {
                    break;
                }

                j++;
            }

            // Remove out-of-range placeholders
            for (var x = 0; x < cache.length; x++) {
                cache[x].remove();
            }

            // Resize list
            if (l > 0) {
                this.list.css(this.wh, this.dimension(this.list) + l + 'px');

                if (back) {
                    pos -= l;
                    this.list.css(this.lt, $jc.intval(this.list.css(this.lt)) - l + 'px');
                }
            }

            // Calculate first and last item
            var last = i + visible - 1;
            if (this.options.wrap != 'circular' && this.options.size && last > this.options.size) {
                last = this.options.size;
            }

            if (j > last) {
                visible = 0;
                j = last;
                v = 0;
                while (++visible) {
                    e = this.get(j--);
                    if (!e.length) {
                        break;
                    }
                    v += this.dimension(e);
                    if (v >= clipping) {
                        break;
                    }
                }
            }

            var first = last - visible + 1;
            if (this.options.wrap != 'circular' && first < 1) {
                first = 1;
            }

            if (this.inTail && back) {
                pos += this.tail;
                this.inTail = false;
            }

            this.tail = null;
            if (this.options.wrap != 'circular' && last == this.options.size && (last - visible + 1) >= 1) {
                var m = $jc.intval(this.get(last).css(!this.options.vertical ? 'marginRight' : 'marginBottom'));
                if ((v - m) > clipping) {
                    this.tail = v - clipping - m;
                }
            }

            if (fv && i === this.options.size && this.tail) {
                pos -= this.tail;
                this.inTail = true;
            }

            // Adjust position
            while (i-- > first) {
                pos += this.dimension(this.get(i));
            }

            // Save visible item range
            this.prevFirst = this.first;
            this.prevLast = this.last;
            this.first = first;
            this.last = last;

            return pos;
        },

        /**
         * Animates the carousel to a certain position.
         *
         * @method animate
         * @return undefined
         * @param p {Number} Position to scroll to.
         * @param a {Boolean} Flag indicating whether to perform animation.
         */
        animate: function (p, a) {
            if (this.locked || this.animating) {
                return;
            }

            this.animating = true;

            var self = this;
            var scrolled = function () {
                self.animating = false;

                if (p === 0) {
                    self.list.css(self.lt, 0);
                }

                if (!self.autoStopped && (self.options.wrap == 'circular' || self.options.wrap == 'both' || self.options.wrap == 'last' || self.options.size === null || self.last < self.options.size || (self.last == self.options.size && self.tail !== null && !self.inTail))) {
                    self.startAuto();
                }

                self.buttons();
                self.notify('onAfterAnimation');

                // This function removes items which are appended automatically for circulation.
                // This prevents the list from growing infinitely.
                if (self.options.wrap == 'circular' && self.options.size !== null) {
                    for (var i = self.prevFirst; i <= self.prevLast; i++) {
                        if (i !== null && !(i >= self.first && i <= self.last) && (i < 1 || i > self.options.size)) {
                            self.remove(i);
                        }
                    }
                }
            };

            this.notify('onBeforeAnimation');

            // Animate
            if (!this.options.animation || a === false) {
                this.list.css(this.lt, p + 'px');
                scrolled();
            } else {
                var o = !this.options.vertical ? (this.options.rtl ? { 'right': p } : { 'left': p }) : { 'top': p };
                // Define animation settings.
                var settings = {
                    duration: this.options.animation,
                    easing: this.options.easing,
                    complete: scrolled
                };
                // If we have a step callback, specify it as well.
                if ($.isFunction(this.options.animationStepCallback)) {
                    settings.step = this.options.animationStepCallback;
                }
                // Start the animation.
                this.list.animate(o, settings);
            }
        },

        /**
         * Starts autoscrolling.
         *
         * @method auto
         * @return undefined
         * @param s {Number} Seconds to periodically autoscroll the content.
         */
        startAuto: function (s) {
            if (s !== undefined) {
                this.options.auto = s;
            }

            if (this.options.auto === 0) {
                return this.stopAuto();
            }

            if (this.timer !== null) {
                return;
            }

            this.autoStopped = false;

            var self = this;
            this.timer = window.setTimeout(function () { self.next(); }, this.options.auto * 1000);
        },

        /**
         * Stops autoscrolling.
         *
         * @method stopAuto
         * @return undefined
         */
        stopAuto: function () {
            this.pauseAuto();
            this.autoStopped = true;
        },

        /**
         * Pauses autoscrolling.
         *
         * @method pauseAuto
         * @return undefined
         */
        pauseAuto: function () {
            if (this.timer === null) {
                return;
            }

            window.clearTimeout(this.timer);
            this.timer = null;
        },

        /**
         * Sets the states of the prev/next buttons.
         *
         * @method buttons
         * @return undefined
         */
        buttons: function (n, p) {
            if (n == null) {
                n = !this.locked && this.options.size !== 0 && ((this.options.wrap && this.options.wrap != 'first') || this.options.size === null || this.last < this.options.size);
                if (!this.locked && (!this.options.wrap || this.options.wrap == 'first') && this.options.size !== null && this.last >= this.options.size) {
                    n = this.tail !== null && !this.inTail;
                }
            }

            if (p == null) {
                p = !this.locked && this.options.size !== 0 && ((this.options.wrap && this.options.wrap != 'last') || this.first > 1);
                if (!this.locked && (!this.options.wrap || this.options.wrap == 'last') && this.options.size !== null && this.first == 1) {
                    p = this.tail !== null && this.inTail;
                }
            }

            var self = this;

            if (this.buttonNext.size() > 0) {
                this.buttonNext.unbind(this.options.buttonNextEvent + '.jcarousel', this.funcNext);

                if (n) {
                    this.buttonNext.bind(this.options.buttonNextEvent + '.jcarousel', this.funcNext);
                }

                this.buttonNext[n ? 'removeClass' : 'addClass'](this.className('jcarousel-next-disabled')).attr('disabled', n ? false : true);

                if (this.options.buttonNextCallback !== null && this.buttonNext.data('jcarouselstate') != n) {
                    this.buttonNext.each(function () { self.options.buttonNextCallback(self, this, n); }).data('jcarouselstate', n);
                }
            } else {
                if (this.options.buttonNextCallback !== null && this.buttonNextState != n) {
                    this.options.buttonNextCallback(self, null, n);
                }
            }

            if (this.buttonPrev.size() > 0) {
                this.buttonPrev.unbind(this.options.buttonPrevEvent + '.jcarousel', this.funcPrev);

                if (p) {
                    this.buttonPrev.bind(this.options.buttonPrevEvent + '.jcarousel', this.funcPrev);
                }

                this.buttonPrev[p ? 'removeClass' : 'addClass'](this.className('jcarousel-prev-disabled')).attr('disabled', p ? false : true);

                if (this.options.buttonPrevCallback !== null && this.buttonPrev.data('jcarouselstate') != p) {
                    this.buttonPrev.each(function () { self.options.buttonPrevCallback(self, this, p); }).data('jcarouselstate', p);
                }
            } else {
                if (this.options.buttonPrevCallback !== null && this.buttonPrevState != p) {
                    this.options.buttonPrevCallback(self, null, p);
                }
            }

            this.buttonNextState = n;
            this.buttonPrevState = p;
        },

        /**
         * Notify callback of a specified event.
         *
         * @method notify
         * @return undefined
         * @param evt {String} The event name
         */
        notify: function (evt) {
            var state = this.prevFirst === null ? 'init' : (this.prevFirst < this.first ? 'next' : 'prev');

            // Load items
            this.callback('itemLoadCallback', evt, state);

            if (this.prevFirst !== this.first) {
                this.callback('itemFirstInCallback', evt, state, this.first);
                this.callback('itemFirstOutCallback', evt, state, this.prevFirst);
            }

            if (this.prevLast !== this.last) {
                this.callback('itemLastInCallback', evt, state, this.last);
                this.callback('itemLastOutCallback', evt, state, this.prevLast);
            }

            this.callback('itemVisibleInCallback', evt, state, this.first, this.last, this.prevFirst, this.prevLast);
            this.callback('itemVisibleOutCallback', evt, state, this.prevFirst, this.prevLast, this.first, this.last);
        },

        callback: function (cb, evt, state, i1, i2, i3, i4) {
            if (this.options[cb] == null || (typeof this.options[cb] != 'object' && evt != 'onAfterAnimation')) {
                return;
            }

            var callback = typeof this.options[cb] == 'object' ? this.options[cb][evt] : this.options[cb];

            if (!$.isFunction(callback)) {
                return;
            }

            var self = this;

            if (i1 === undefined) {
                callback(self, state, evt);
            } else if (i2 === undefined) {
                this.get(i1).each(function () { callback(self, this, i1, state, evt); });
            } else {
                var call = function (i) {
                    self.get(i).each(function () { callback(self, this, i, state, evt); });
                };
                for (var i = i1; i <= i2; i++) {
                    if (i !== null && !(i >= i3 && i <= i4)) {
                        call(i);
                    }
                }
            }
        },

        create: function (i) {
            return this.format('<li></li>', i);
        },

        format: function (e, i) {
            e = $(e);
            var split = e.get(0).className.split(' ');
            for (var j = 0; j < split.length; j++) {
                if (split[j].indexOf('jcarousel-') != -1) {
                    e.removeClass(split[j]);
                }
            }
            e.addClass(this.className('jcarousel-item')).addClass(this.className('jcarousel-item-' + i)).css({
                'float': (this.options.rtl ? 'right' : 'left'),
                'list-style': 'none'
            }).attr('jcarouselindex', i);
            return e;
        },

        className: function (c) {
            return c + ' ' + c + (!this.options.vertical ? '-horizontal' : '-vertical');
        },

        dimension: function (e, d) {
            var el = $(e);

            if (d == null) {
                return !this.options.vertical ?
                    ((el.innerWidth() +
                        $jc.intval(el.css('margin-left')) +
                        $jc.intval(el.css('margin-right')) +
                        $jc.intval(el.css('border-left-width')) +
                        $jc.intval(el.css('border-right-width'))) || $jc.intval(this.options.itemFallbackDimension)) :
                    ((el.innerHeight() +
                        $jc.intval(el.css('margin-top')) +
                        $jc.intval(el.css('margin-bottom')) +
                        $jc.intval(el.css('border-top-width')) +
                        $jc.intval(el.css('border-bottom-width'))) || $jc.intval(this.options.itemFallbackDimension));
            } else {
                var w = !this.options.vertical ?
                    d - $jc.intval(el.css('marginLeft')) - $jc.intval(el.css('marginRight')) :
                    d - $jc.intval(el.css('marginTop')) - $jc.intval(el.css('marginBottom'));

                $(el).css(this.wh, w + 'px');

                return this.dimension(el);
            }
        },

        clipping: function () {
            return !this.options.vertical ?
                this.clip[0].offsetWidth - $jc.intval(this.clip.css('borderLeftWidth')) - $jc.intval(this.clip.css('borderRightWidth')) :
                this.clip[0].offsetHeight - $jc.intval(this.clip.css('borderTopWidth')) - $jc.intval(this.clip.css('borderBottomWidth'));
        },

        index: function (i, s) {
            if (s == null) {
                s = this.options.size;
            }

            return Math.round((((i - 1) / s) - Math.floor((i - 1) / s)) * s) + 1;
        }
    });

    $jc.extend({
        /**
         * Gets/Sets the global default configuration properties.
         *
         * @method defaults
         * @return {Object}
         * @param d {Object} A set of key/value pairs to set as configuration properties.
         */
        defaults: function (d) {
            return $.extend(defaults, d || {});
        },

        intval: function (v) {
            v = parseInt(v, 10);
            return isNaN(v) ? 0 : v;
        },

        windowLoaded: function () {
            windowLoaded = true;
        },

        isSafari: function () {
            var ua = navigator.userAgent.toLowerCase(),
                match = /(chrome)[ \/]([\w.]+)/.exec(ua) || /(webkit)[ \/]([\w.]+)/.exec(ua) || [],
                browser = match[1] || "";

            return browser === 'webkit';
        }
    });

    /**
     * Creates a carousel for all matched elements.
     *
     * @example $("#mycarousel").jcarousel();
     * @before <ul id="mycarousel" class="jcarousel-skin-name"><li>First item</li><li>Second item</li></ul>
     * @result
     *
     * <div class="jcarousel-skin-name">
     *   <div class="jcarousel-container">
     *     <div class="jcarousel-clip">
     *       <ul class="jcarousel-list">
     *         <li class="jcarousel-item-1">First item</li>
     *         <li class="jcarousel-item-2">Second item</li>
     *       </ul>
     *     </div>
     *     <div disabled="disabled" class="jcarousel-prev jcarousel-prev-disabled"></div>
     *     <div class="jcarousel-next"></div>
     *   </div>
     * </div>
     *
     * @method jcarousel
     * @return jQuery
     * @param o {Hash|String} A set of key/value pairs to set as configuration properties or a method name to call on a formerly created instance.
     */
    $.fn.jcarousel = function (o) {
        if (typeof o == 'string') {
            var instance = $(this).data('jcarousel'), args = Array.prototype.slice.call(arguments, 1);
            return instance[o].apply(instance, args);
        } else {
            return this.each(function () {
                var instance = $(this).data('jcarousel');
                if (instance) {
                    if (o) {
                        $.extend(instance.options, o);
                    }
                    instance.reload();
                } else {
                    $(this).data('jcarousel', new $jc(this, o));
                }
            });
        }
    };

})(jQuery);





/******** from legacy jquery-scrollable.js file ****************/

(function ($) {
    "";//yui compressor fix
    "use strict";
    /**
	 * Creates a slideshow for all matched elements.
	 *
	 * @example $("#pikame").PikaChoose();
	 * @input needed <ul id="pikame"><li><img src="first.jpg"><span>Caption</span></li><li><img src="second.jpg"><span>Caption</span></li></ul>
	 *
	 * @method PikaChoose
	 * @return $
	 * @param o {Hash|String} A set of key/value pairs to set as configuration properties or a method name to call on a formerly created instance.
	 */
    var imgloaded = function (evt) { $(this).data("loaded", true); };
    var imgloaderror = function (evt) { $(this).data("error", true); };

    var defaults = {
        autoPlay: true,
        speed: 5000,
        text: { play: "", stop: "", previous: "Previous", next: "Next", loading: "Loading" },
        transition: [1],
        showCaption: true,
        IESafe: false,
        showTooltips: false,
        carousel: false,
        swipe: true,
        carouselVertical: false,
        animationFinished: null,
        buildFinished: null,
        bindsFinished: null,
        startOn: 0,
        thumbOpacity: 0.4,
        hoverPause: false,
        animationSpeed: 600,
        fadeThumbsIn: true, /* default: false */
        carouselOptions: {},
        thumbChangeEvent: "click.pikachoose",
        stopOnClick: false,
        hideThumbnails: false
        /* Custom Variables */
		, carouselOrientation: "bottom"
		, carouselBackgroundColor: "" /* Added 1-9-2012 by ltylenda */
		, showText: false
		, captionOpacity: 0.7
		, captionOrientation: "bottom"
		, captionColor: "000000"
		, captionTextColor: "ffffff"
		, captionTextSize: 11
		, captionTextFont: ""
		, captionTextAlignment: "left"
		, captionTextClass: "caption-text-default"
        /*, captionTransition: "none" // Not Built */
		, showManualControls: true
		, manualControlsType: "default"
        /*, fullscreen: false // Not Built */
		, shuffleItems: false
		, mainItemWidth: 640
		, mainItemHeight: 420
		, mainItemBackgroundColor: "" /* Added 1-9-2012 by ltylenda */
        /*, cropMainItem: false // Not Built
		, cropThumbs: false // Not Built
		, cropMainItemOrientation: "none" // Not Built
		, cropThumbOrientation: "none" // Not Built */
		, thumbWidth: 75
		, thumbHeight: 75
        /*, mainItemPadding: [0, 0, 0, 0] // Not Built */
		, thumbSpacing: 10
		, marginCarouselMainItem: 0
        /* End Custom Variables */
    };

    $.fn.PikaChoose = function (o) {
        return this.each(function () {
            $(this).data('pikachoose', new $pc(this, o));
        });
    };

    /**
	 * The PikaChoose object.
	 *
	 * @constructor
	 * @class pikachoose
	 * @param e {HTMLElement} The element to create the carousel for.
	 * @param o {Object} A set of key/value pairs to set as configuration properties.
	 * @cat Plugins/PikaChoose
	 */
    $.PikaChoose = function (e, o) {
        this.options = $.extend({}, defaults, o || {});
        this.list = null;
        this.image = null;
        this.anchor = null;
        this.caption = null;
        this.imgNav = null;
        this.imgPlay = null;
        this.imgPrev = null;
        this.imgNext = null;
        this.textNext = null;
        this.textPrev = null;
        this.previous = null;
        this.next = null;
        this.aniWrap = null;
        this.aniDiv = null;
        this.aniImg = null;
        this.thumbs = null;
        this.transition = null;
        this.active = null;
        this.tooltip = null;
        this.animating = false;
        this.stillOut = null;
        this.counter = null;
        this.timeOut = null;
        this.pikaStage = null;
        /* My Code
		Extended functionality */
        this.galleryType = "bottom";
        this.manualControls = null;
        this.intDifference = 0;
        this.captionBackground = null;
        this.captionText = null;
        this.captionInner = null;
        this.usePagination = this.options.manualControlsType == "number" || this.options.manualControlsType == "dot";
        this.showManualControlsWithNoPagination = this.options.showManualControls && this.usePagination !== true;
        /* End Extended functionality */
        this.dfdLoaded = $.Deferred();
        this.dfdLoaded.done(function () { });
        /* To render correct gallery css */
        if (this.options.carousel && $.fn.jcarousel) {
            this.galleryType = this.options.carouselOrientation;
            var removeWidth = this.options.carouselVertical === true;
            /* Find out the size of the carousel according to its position */
            var intMainItemDimensionRemove = (removeWidth ? this.options.thumbWidth : this.options.thumbHeight) + 15 * 2;
            var intMainItemDimension = removeWidth ? this.options.mainItemWidth : this.options.mainItemHeight;
            if (intMainItemDimension > intMainItemDimensionRemove) {
                intMainItemDimension -= intMainItemDimensionRemove;
            }

            if (removeWidth) {
                this.options.mainItemWidth = intMainItemDimension;
            }
            else {
                this.options.mainItemHeight = intMainItemDimension;
            }
        }
        else {
            this.options.hideThumbnails = true;
            if (this.options.showManualControls === true) {
                if (this.options.showCaption !== true) {
                    this.galleryType = "sliding";
                }
            }
            else {
                if (this.options.autoPlay === true) {
                    this.galleryType = "slideshow";
                }
            }
        }

        var pikaInit = $(e);
        if (pikaInit.hasClass("pikachoose") === false) {
            pikaInit = pikaInit.closest(".pikachoose");
        }
        pikaInit.width(parseInt(this.options.mainItemWidth, 10))
			.addClass("pika-scrollable-" + this.galleryType);
        /* End My Code */

        if (typeof (this.options.data) != "undefined") {
            var self = this;
            //user passed a data source
            e = $('<ul></ul>').toggleClass("jcarousel-skin-pika", this.options.jcarousel).appendTo(e);
            $.each(this.options.data, function () {
                var tmp = null;
                if (typeof (this.link) != "undefined") {
                    tmp = $("<li><a href='" + this.link + "'><img></a></li>").appendTo(e);
                    if (typeof (this.title) != "undefined") { tmp.find('a').attr('title', this.title); }
                } else {
                    tmp = $("<li><img></li>").appendTo(e);
                }
                /* My Code */
                // put tag saying image is loaded and on error, show noimage placeholder
                tmp.find('img').onload = imgloaded;
                tmp.find('img').onerror = imgloaderror;
                /* End My Code */
                if (typeof (this.caption) != "undefined") { tmp.append("<span>" + this.caption + "</span>"); }
                if (typeof (this.thumbnail) != "undefined" && self.options.carousel) {
                    tmp.find('img').attr('ref', this.image);
                    tmp.find('img').attr('src', this.thumbnail);
                } else {
                    tmp.find('img').attr('src', this.image);
                }
            });
            /* My Code */
        } else {
            // put tag saying image is loaded and on error, show noimage placeholder
            $(e).find('img').each(function (i) {
                this.onload = imgloaded;
                this.onerror = imgloaderror;
            });
        }
        /* End My Code */
        if (e instanceof jQuery || e.nodeName.toUpperCase() == 'UL' || e.nodeName.toUpperCase() == 'OL') {
            this.list = $(e);
            this.build();
            this.bindEvents();
            /* My Code */
            this.dfdLoaded.resolve("");
            /* End My Code */
        } else {
            this.dfdLoaded.reject("");
            return;
        }

        var y = 0;
        var x = 0;
        for (var t = 0; t < 25; t++) {
            var a = '<div col="' + y + '" row="' + x + '"></div>';
            this.aniDiv.append(a);
            y++;
            if (y == 5) {
                x++;
                y = 0;
            }
        }
    };//end PikaChoose function(e, o)

    var $pc = $.PikaChoose;
    $pc.fn = $pc.prototype = {
        version: "96"
    };

    $.fn.pikachoose = $.fn.PikaChoose;

    $pc.fn.extend = $pc.extend = $.extend;

    $pc.fn.extend({
        /* My Code */
        getHexColor: function (_color, _default) {
            if (!!_color) {
                _color = _color.replace("#", "");
            }
            else {
                _color = "";
            }
            if (_default === undefined) {
                _default = "transparent";
            }
            return (_color && (_color.length == 6 || _color.length == 3)) ? "#" + _color : _default;
        },
        /* End My Code */
        /**
		 * Builds the gallery structure.
		 *
		 * @method build
		 * @return undefined
		 */
        build: function () {
            this.step = 0; //transition step count
            //create the structure for pikachoose
            if (this.options.pikaStage) {
                this.wrap = this.options.pikaStage;
                this.wrap.addClass('pika-stage');
            }
            else {
                this.wrap = $("<div class='pika-stage'></div>").insertBefore(this.list);
            }
            /* My Code */
            // Set width and height of main item
            this.wrap
				.width(this.options.mainItemWidth)
				.height(this.options.mainItemHeight)
				.css("background-color", this.getHexColor(this.options.mainItemBackgroundColor, "transparent"));
            /* End My Code */
            this.image = $("<img>").appendTo(this.wrap);
            this.imgNav = $("<div class='pika-imgnav'></div>").hide().insertAfter(this.image);
            this.imgPlay = $("<a></a>").appendTo(this.imgNav);
            this.counter = $("<span class='pika-counter'></span>").appendTo(this.imgNav);
            if (this.options.autoPlay) { this.imgPlay.addClass('pause'); } else { this.imgPlay.addClass('play'); }
            this.imgPrev = $("<a class='previous'></a>").insertAfter(this.imgPlay);
            this.imgNext = $("<a class='next'></a>").insertAfter(this.imgPrev);
            this.caption = $("<div class='caption'></div>").insertAfter(this.imgNav).hide();
            /* My Code */
            this.captionInner = $("<div class='caption-inner'></div>").appendTo(this.caption);
            this.captionBackground = $("<div class='caption-background'></div>").appendTo(this.captionInner);
            this.captionText = $("<div class='caption-text'></div>").insertAfter(this.captionBackground);
            this.tooltip = $("<div class='pika-tooltip'></div>").insertAfter(this.list).hide();

            /* Set the caption properties */
            if (this.options.showCaption === true) {
                this.captionBackground.css({
                    "background-color": this.getHexColor(this.options.captionColor, "000"),
                    "opacity": this.options.captionOpacity
                });

                var captionCss = {
                    "position": "absolute",
                    "border": 0,
                    "font-size": this.options.captionTextSize + "pt",
                    "color": this.getHexColor(this.options.captionTextColor, "fff"),
                    "padding": 0,
                    "text-align": this.options.captionTextAlignment
                };

                if (!!this.options.captionTextFont) {
                    captionCss["font-family"] = this.options.captionTextFont;
                }

                var cssObj = {
                    top: 0,
                    bottom: 0,
                    right: 0,
                    left: 0,
                    width: "20%"
                };
                switch (this.options.captionOrientation.toLowerCase()) {
                    case "bottom":
                        cssObj.top = undefined;
                        cssObj.width = undefined;
                        break;
                    case "top":
                        cssObj.bottom = undefined;
                        cssObj.width = undefined;
                        break;
                    case "left":
                        cssObj.right = undefined;
                        break;
                    case "right":
                        cssObj.left = undefined;
                        break;
                    default:
                        cssObj = {};
                        break;
                }

                this.caption.css($.extend(captionCss, cssObj));
            }
            /* End My Code */
            this.aniWrap = $("<div class='pika-aniwrap'></div>").insertAfter(this.caption);
            this.aniImg = $("<img>").appendTo(this.aniWrap).hide();
            this.aniDiv = $("<div class='pika-ani'></div>").appendTo(this.aniWrap);
            this.textNav = $("<div class='pika-textnav'></div>").insertAfter(this.aniWrap);
            this.textPrev = $("<a class='previous'>" + this.options.text.previous + "</a>").appendTo(this.textNav);
            this.textNext = $("<a class='next'>" + this.options.text.next + "</a>").appendTo(this.textNav);
            this.list.addClass("pika-thumbs");
            this.thumbs = this.list.find("img");
            this.thumbs.each(function (i) {
                this.onload = imgloaded;
                this.onerror = imgloaderror;
            });
            this.loader = $("<div class='pika-loader'></div>").appendTo(this.wrap).hide().html(this.options.text.loading);
            this.active = this.thumbs.eq(this.options.startOn);
            /* My Code */
            if (this.usePagination) {
                this.manualControls = $("<ul class='pagination'></ul>")
					.insertAfter(this.list)
					.width(this.options.mainItemWidth);

                // for each slide create a list item and link
                var item = "<a class='" + (this.options.manualControlsType === "number" ? "numberControls" : "dotControls") + "'></a>";
                for (var number = 0; number < this.thumbs.length; number++) {
                    $(item).attr("href", "#" + number).text(number + 1).appendTo(this.manualControls).wrap("<li></li>");
                }

                // add current class to start slide pagination
                this.manualControls.find("li")
					.eq(this.options.startOn)
					.addClass("currentItem");
            }
            /* End My Code */
            //fill in info for first image
            this.finishAnimating({
                'index': this.options.startOn,
                'source': this.active.attr('ref') || this.active.attr('src'),
                'caption': this.active.parents('li:first').find('span:first').html(),
                'clickThrough': this.active.parent().attr('href') || "",
                'clickThroughTarget': this.active.parent().attr('target') || "",
                'clickThroughTitle': this.active.parent().attr('title') || ""
                /* My Code */
				, 'width': this.options.mainItemWidth || 0,
                'height': this.options.mainItemHeight || 0
                /* End My Code */
            });
            //Set CSS that is a MUST have. All style specific stuff happens in css file
            this.aniDiv.css({ position: "relative" });

            //process all the thumbnails
            var self = this;
            this.updateThumbs();
            //thubnails must fade in before jcarousel is called or we get that awesome no width/height error
            if (this.options.fadeThumbsIn) {
                this.list.fadeIn();
            }

            if (this.options.hideThumbnails) {
                this.list.hide();
            }
            if (this.options.carousel && $.fn.jcarousel) {
                /* My Code */
                var blnIsVertical = (this.options.carouselOrientation == "left" || this.options.carouselOrientation == "right");
                // set thumb spacing
                var thumbCss = {
                    "background-color": "transparent",
                    border: 0,
                    width: this.options.thumbWidth + "px",
                    height: this.options.thumbHeight + "px",
                    "line-height": this.options.thumbHeight + "px"
                };
                if (!blnIsVertical) {
                    thumbCss["margin-left"] = 0;
                }
                this.list.find("li")
					.css(thumbCss)
					.css("margin-" + (blnIsVertical ? "bottom" : " right"), this.options.thumbSpacing + "px");

                var intCarouselWidth = parseInt(this.options.mainItemWidth, 10);
                var intCarouselHeight = parseInt(this.options.mainItemHeight, 10);
                //var intCarouselOuterWidth, intCarouselOuterHeight;
                /* End My Code */
                //default options for carousel
                var carouselDefaults = {
                    vertical: blnIsVertical /*this.options.carouselVertical*/,
                    initCallback: function (carousel) {
                        jQuery(carousel.list).find('img').click(function (e, x) {
                            if (typeof (x) !== 'undefined' && x.how == "auto") {
                                if (self.options.autoPlay === false) {
                                    return false;
                                }
                            }
                            var clicked = parseInt(jQuery(this).parents('.jcarousel-item').attr('jcarouselindex'), 10);
                            var last = (jQuery(this).parents('ul').find('li:last').attr('jcarouselindex') == clicked - 1) ? true : false;
                            if (!last) {
                                clicked = (clicked - 2 <= 0) ? 0 : clicked - 2;
                            }
                            clicked++;
                            carousel.scroll(clicked);
                        });
                        /* My Code */
                        // Adjust carousel size
                        if (carousel.options.vertical) {
                            carousel.container.outerHeight(intCarouselHeight);
                        }
                        else {
                            carousel.container.outerWidth(intCarouselWidth);
                        }
                        carousel.container.css("background-color", self.getHexColor(self.options.carouselBackgroundColor, "transparent"));
                        /* End My Code */
                    }
                };
                var carouselOptions = $.extend({}, carouselDefaults, this.options.carouselOptions || {});

                this.list.jcarousel(carouselOptions);
                var carousel = $(this.list).data("jcarousel");
                var offset = (this.options.carouselVertical ? ((carousel.container.outerWidth() / 2) - (carousel.buttonPrev.width() / 2)) : ((carousel.container.outerHeight() / 2) - (carousel.buttonPrev.height() / 2))) + "px";
                var direction = this.options.carouselVertical ? "left" : "top";
                carousel.buttonPrev.css(direction, offset);
                carousel.buttonNext.css(direction, offset);

                /* My Code */
                // space between carousel and main image
                switch (this.options.carouselOrientation.toLowerCase()) {
                    case 'bottom':
                        carousel.container
							.closest('.jcarousel-skin-pika')
							.css('margin-top', this.options.marginCarouselMainItem + 'px');
                        break;
                    case 'top':
                        this.wrap
							.css('padding-top', (carousel.container.outerHeight() + this.options.marginCarouselMainItem) + 'px');
                        break;
                    case 'left':
                        this.wrap
							.css('left', (carousel.container.outerWidth() + this.options.marginCarouselMainItem) + 'px');
                        break;
                    case 'right':
                        carousel.container
							.closest('.jcarousel-skin-pika')
							.css('left', (this.options.mainItemWidth + this.options.marginCarouselMainItem) + 'px');
                        break;
                }

                if (this.galleryType == 'bottom') {
                    var paddingTop = (-this.options.marginCarouselMainItem) + 'px';
                    $(this.imgPrev).css('padding-top', paddingTop);
                    $(this.imgNext).css('padding-top', paddingTop);
                }
                /* End My Code */
            }

            if (this.options.swipe && $.fn.touchwipe) {
                this.wrap.touchwipe({
                    wipeLeft: function () {
                        self.Next();
                    },
                    wipeRight: function () {
                        self.Prev();
                    }
                });
            }
            /* My Code */
            this.wrap.children().wrapAll("<div class='pika-stage-inner'></div>");
            /* End My Code */
            if (typeof (this.options.buildFinished) == 'function') {
                this.options.buildFinished(this);
            }
        }, //end setup
        /**
		 * proccesses thumbnails
		 *
		 * @method createThumb
		 * @return undefined
		 */
        createThumb: function (ele) {
            var self = ele;
            var that = this;
            this.thumbs = this.list.find("img");
            if (typeof $.data(ele[0], 'source') !== 'undefined') { return; }
            ele.parents('li:first').wrapInner("<div class='clip' />");
            self.hide();
            //store all the data with the image
            $.data(ele[0], 'clickThrough', self.parent('a').attr('href') || "");
            $.data(ele[0], 'clickThroughTarget', self.parent('a').attr('target') || "");
            $.data(ele[0], 'clickThroughTitle', self.parent('a').attr('title') || "");
            if (self.parent('a').length > 0) { self.unwrap(); }
            $.data(ele[0], 'caption', self.next('span').html() || "");
            self.next('span').remove();
            $.data(ele[0], 'source', self.attr('ref') || self.attr('src'));
            $.data(ele[0], 'imageTitle', self.attr('title') || "");
            $.data(ele[0], 'imageAlt', self.attr('alt') || "");
            $.data(ele[0], 'index', this.thumbs.index(ele));

            //gets each items index to iterate through them. Thanks to Tushar for the fix.
            $.data(ele[0], 'order', self.closest('ul').find('li').index(self.parents('li')));
            //pass data so it can enter the load scope
            var data = $.data(ele[0]);
            $('<img />').on('load', { data: data }, function () {
                if (typeof (that.options.buildThumbStart) == 'function') {
                    that.options.buildThumbStart(that);
                }
                //in this scope self refers to the image
                var img = $(this);
                var w = this.width;
                var h = this.height;
                if (w === 0) { w = img.attr("width"); }
                if (h === 0) { h = img.attr("height"); }
                //grab a ratio for image to user defined settings
                /* My Code */
                var ph = that.options.carousel ? that.options.thumbWidth : that.options.mainItemWidth;
                var pw = that.options.carousel ? that.options.thumbHeight : that.options.mainItemHeight;
                /* End My Code */
                /* ORIGINAL CODE
				var ph = parseInt(self.parents('.clip').css('height').slice(0, -2), 10);
				var pw = parseInt(self.parents('.clip').css('width').slice(0, -2), 10);				
				*/
                if (pw === 0) {
                    //failsafe for IE8
                    pw = self.parents('li:first').outerWidth();
                }
                if (ph === 0) {
                    //failsafe for IE8
                    ph = self.parents('li:first').outerHeight();
                }
                //ratio width/height
                var rw = pw / w;
                var rh = ph / h;
                //determine which has the smallest ratio (thus needing
                //to be the side we use to scale so our whole thumb is filled)
                var ratio;
                /* My Code	*/
                if (rw > 1 && rh > 1) {
                    ratio = 1;
                }
                else {
                    //we'll use ratio later to scale and not distort
                    ratio = rw > rh ? rh : rw;
                }
                /* End My Code */
                /* ORIGINAL CODE
				if (rw < rh)
				{
					//we'll use ratio later to scale and not distort
					self.css({ height: '100%' });
					self.fadeTo(0.001, 0);
					self.css({ left: -(self.outerWidth() - pw) / 2 });
					self.hide();
				}
				else
				{
					self.css({ width: '100%' });
				}
				*/
                //use those ratios to calculate scale
                /* My Code */
                var width = Math.round(w * ratio);
                var height = Math.round(h * ratio);
                self.attr({
                    width: width,
                    height: height
                })
					.css("position", "relative");
                /* End My Code */
                self.hover(
					function (e) {
					    clearTimeout(that.stillOut);

					    // Center landscape images. Have to blink it into existence for a second to get an accuracte width
					    $(this).stop(true, true).fadeTo(250, 1);
					    if (!that.options.showTooltips) { return; }
					    that.tooltip.show().stop(true, true).html(data.caption).animate({ top: $(this).parent().position().top, left: $(this).parent().position().left, opacity: 1.0 }, 'fast');
					},
					function (e) {
					    if (!$(this).hasClass("active")) {
					        $(this).stop(true, true).fadeTo(250, that.options.thumbOpacity);
					        that.stillOut = setTimeout(that.hideTooltip, 700);
					    }
					}
				);

                if (data.order == that.options.startOn) {
                    self.fadeTo(250, 1);
                    self.addClass('active');
                    self.parents('li').eq(0).addClass('active');
                } else {
                    self.fadeTo(250, that.options.thumbOpacity);
                }
                if (typeof (that.options.buildThumbFinish) == 'function') {
                    that.options.buildThumbFinish(that);
                }
            }).attr('src', self.attr('src'));
        },//end createThumb
        /**
		 * proccesses thumbnails
		 *
		 * @method bindEvents
		 * @return undefined
		 */
        bindEvents: function () {
            this.thumbs.on(this.options.thumbChangeEvent, { self: this }, this.imgClick);
            this.imgNext.on('click.pikachoose', { self: this }, this.nextClick);
            this.textNext.on('click.pikachoose', { self: this }, this.nextClick);
            this.imgPrev.on('click.pikachoose', { self: this }, this.prevClick);
            this.textPrev.on('click.pikachoose', { self: this }, this.prevClick);
            this.imgPlay.unbind('click.pikachoose').on('click.pikachoose', { self: this }, this.playClick);
            this.wrap.unbind('mouseenter.pikachoose').on('mouseenter.pikachoose', { self: this }, function (e) {
                /* My Code. if condition */
                if (e.data.self.showManualControlsWithNoPagination) {
                    e.data.self.imgNav.stop(true, true).fadeTo('slow', 1.0);
                }
                if (e.data.self.options.hoverPause === true) {
                    clearTimeout(e.data.self.timeOut);
                }
            });
            this.wrap.unbind('mouseleave.pikachoose').on('mouseleave.pikachoose', { self: this }, function (e) {
                /* My Code. if condition */
                if (e.data.self.showManualControlsWithNoPagination) {
                    e.data.self.imgNav.stop(true, true).fadeTo('slow', 0.0);
                }
                if (e.data.self.options.autoPlay && e.data.self.options.hoverPause) {
                    e.data.self.timeOut = setTimeout((function (self) {
                        return function () { self.nextClick(); };
                    })(e.data.self), e.data.self.options.speed);
                }
            });
            this.tooltip.unbind('mouseenter.pikachoose').on('mouseenter.pikachoose', { self: this }, function (e) {
                clearTimeout(e.data.self.stillOut);
            });
            this.tooltip.unbind('mouseleave.pikachoose').on('mouseleave.pikachoose', { self: this }, function (e) {
                e.data.self.stillOut = setTimeout(e.data.self.hideTooltip, 700);
            });

            /* My Code */
            if (this.manualControls) {
                // click handling
                this.manualControls.find("li").on('click.pikachoose', { self: this }, function (e) {
                    // pause slideshow
                    if (e.data.self.options.autoPlay) {
                        clearTimeout(e.data.self.timeOut);
                        e.data.self.wrap.trigger('mouseenter.pikachoose');
                    }
                    // get clicked, pass to animate function
                    var clicked = e.data.self.manualControls.find("li").index(this);
                    var current = e.data.self.list.find("img").index(e.data.self.active);

                    // if current slide equals clicked, don't do anything
                    if (current != clicked) {
                        // get next from pagination item clicked, convert to number
                        var next = parseInt(clicked, 10);
                        // get previous from pagination item with class of current
                        var prev = e.data.self.manualControls.find('li').index('.currentItem');
                        // set current state for pagination
                        // remove current class from all
                        e.data.self.intDifference = next - prev;
                        e.data.self.manualControls
							.find('li')
							.filter('.currentItem')
							.removeClass('currentItem')
							.end()
						// add current class to next
							.eq(next)
							.addClass('currentItem');
                        e.data.self.GoTo(next);
                    }
                });
            }
            /* End My Code */
            if (typeof (this.options.bindsFinished) == 'function') {
                this.options.bindsFinished(this);
            }
        },//end bind event
        /**
		 * hides tooltip
		 *
		 * @method hideTooltip
		 * @return undefined
		 */
        hideTooltip: function (e) {
            $(".pika-tooltip").animate({ opacity: 0.01 });
        },
        /**
		 * handles gallery after a click occurs. and sets active classes
		 *
		 * @method imgClick
		 * @return undefined
		 */
        imgClick: function (e, x) {
            var self = e.data.self;
            var data = $.data(this);
            if (self.animating) { return; }
            if (typeof (x) == 'undefined' || x.how != "auto") {
                //arrive here if natural click
                if (self.options.autoPlay && self.options.stopOnClick) {
                    self.imgPlay.trigger('click');
                } else {
                    clearTimeout(self.timeOut);
                }
            } else {
                if (!self.options.autoPlay) {
                    return false;
                }
            }
            //check if the src is the same as the current photo or if they're using user thumb
            if ($(this).attr('src') !== $.data(this).source) {
                if ($(this).data("loaded") !== true && $(this).data("error") !== true) {
                    self.loader.fadeIn('fast');
                }
            }
            //self.intDifference = self.getDirection(self, self.list.find("img").index(this));
            self.caption.fadeOut('slow');
            self.animating = true;
            self.active.fadeTo(300, self.options.thumbOpacity).removeClass('active');
            self.active.parents('.active').eq(0).removeClass('active');
            self.active = $(this);
            self.active.addClass('active').fadeTo(200, 1);
            self.active.parents('li').eq(0).addClass('active');
            $('<img />').on('load', { self: self, data: data }, function () {
                self.loader.fadeOut('fast');
                //in this scope self referes to the PikaChoose object
                self.aniDiv.css({ height: self.image.height(), width: self.image.width() }).show();
                self.aniDiv.children('div').css({ 'width': '20%', 'height': '20%', 'float': 'left' });

                /* My Code */
                // Do animation div resizing here and add/update the mainWidth and mainHeight data attributes.
                var rw = self.options.mainItemWidth / this.width,
					rh = self.options.mainItemHeight / this.height,
					ratio = (rw > 1 && rh > 1) ? 1 : (rw > rh ? rh : rw);
                $.extend(data, {
                    mainWidth: Math.round(this.width * ratio),
                    mainHeight: Math.round(this.height * ratio)
                });
                /* End My Code */
                //decide our transition
                var n = 0;
                if (self.options.transition[0] == -1) {
                    //random
                    n = Math.floor(Math.random() * 7) + 1;
                } else {
                    n = self.options.transition[self.step];
                    self.step++;
                    if (self.step >= self.options.transition.length) { self.step = 0; }
                }
                /* made function 'isOldIE' to check for old IE (version 8 and below) */
                if (self.options.IESafe && $pc.isOldIE) { n = 1; }
                self.doAnimation(n, data);

            }).attr('src', $.data(this).source);//end image preload
        },//end bindEvents
        /**
		 * Takes a animation # n and executes it
		 *
		 * @method doAnimation
		 * @return undefined
		 */
        doAnimation: function (n, data) {
            /* ORIGINAL CODE
			// IN CSS base file
			this.aniWrap.css({position:'absolute',top:this.wrap.css('padding-top'), left:this.wrap.css('padding-left'), width: this.wrap.width()});
			*/
            var self = this; //self in this scope refers to PikaChoose object. Needed for callbacks on animations
            self.image.stop(true, false);
            self.caption.stop().fadeOut();
            var aWidth = self.aniDiv.children('div').eq(0).width();
            var aHeight = self.aniDiv.children('div').eq(0).height();
            /* ORIGINAL CODE
			var img = new Image();
			$(img).attr('src', data.source);
			if(img.height != self.image.height() || img.width != self.image.width()){
				//Your images are not the same height? Well you get limited on transitions
				if(n !== 0 && n !== 1 && n !== 7){
					//n = 1;
				}
			}*/
            //self.aniDiv.css({height:self.image.height(),width:self.image.width()});

            /* My Code */
            self.aniDiv.css({ height: data.mainHeight, width: data.mainWidth, top: self.image.position().top + "px", left: self.image.position().left + "px" });
            /* End My Code */
            self.aniDiv.children().each(function () {
                //position myself absolutely
                var div = $(this);
                var xOffset = Math.floor(div.parent().width() / 5) * div.attr('col');
                var yOffset = Math.floor(div.parent().height() / 5) * div.attr('row');
                div.css({
                    'background': 'url(' + data.source + ') -' + xOffset + 'px -' + yOffset + 'px',
                    'background-size': div.parent().width() + 'px ' + div.parent().height() + 'px',
                    'width': '0px',
                    'height': '0px',
                    'position': 'absolute',
                    'top': yOffset + 'px',
                    'left': xOffset + 'px',
                    'float': 'none'
                });
            });//end ani_divs.children.each
            self.aniDiv.hide();
            self.aniImg.hide();

            switch (n) {
                case 0:
                    //fade out then in
                    self.image.stop(true, true).fadeOut(self.options.animationSpeed, function () {
                        /* My Code */
                        //RESIZE FOR DIFFERENT SIZE IMAGE
                        self.imgResize(self.aniDiv, self.image, 0, 0);
                        /* End My Code */
                        self.image.attr('src', data.source).fadeIn(self.options.animationSpeed, function () {
                            self.finishAnimating(data);
                        });
                    });
                    break;
                case 1:
                    //full frame fade
                    self.aniDiv.hide();
                    /* My Code */
                    //RESIZE FOR DIFFERENT SIZE IMAGE
                    self.imgResize(self.aniDiv, self.aniImg, 0, 0);
                    self.aniImg.hide().attr('src', data.source);
                    /* End My Code */
                    /* ORIGINAL CODE */
                    //self.aniImg.height(self.image.height()).hide().attr('src',data.source);
                    //self.wrap.css({height:self.image.height()});
                    $.when(
						self.image.fadeOut(self.options.animationSpeed),
						self.aniImg.eq(0).fadeIn(self.options.animationSpeed)
						).done(function () {
						    self.finishAnimating(data);
						});
                    break;
                case 2:
                    self.aniDiv.show().children().hide().each(function (index) {
                        //animate out as blocks
                        var delay = index * 30;
                        $(this).css({ opacity: 0.1 }).show().delay(delay).animate({ opacity: 1, "width": aWidth, "height": aHeight }, 200, 'linear', function () {
                            if (self.aniDiv.find("div").index(this) == 24) {
                                self.finishAnimating(data);
                            }
                        });
                    });
                    break;
                case 3:
                    self.aniDiv.show().children("div:lt(5)").hide().each(function (index) {
                        var delay = $(this).attr('col') * 100;
                        $(this).css({ opacity: 0.1, "width": aWidth }).show().delay(delay).animate({ opacity: 1, "height": self.image.height() }, self.options.animationSpeed, 'linear', function () {
                            if (self.aniDiv.find(" div").index(this) == 4) {
                                self.finishAnimating(data);
                            }
                        });
                    });
                    break;
                case 4:
                    self.aniDiv.show().children().hide().each(function (index) {
                        if (index > 4) { return; }
                        var delay = $(this).attr('col') * 30;
                        var gap = self.gapper($(this), aHeight, aWidth);
                        var speed = self.options.animationSpeed * 0.7;
                        $(this).css({ opacity: 0.1, "height": "100%" }).show().delay(delay).animate({ opacity: 1, "width": gap.width }, speed, 'linear', function () {
                            if (self.aniDiv.find(" div").index(this) == 4) {
                                self.finishAnimating(data);
                            }
                        });
                    });
                    break;
                case 5:
                    self.aniDiv.show().children().show().each(function (index) {
                        var delay = index * Math.floor(Math.random() * 5) * 7;
                        var gap = self.gapper($(this), aHeight, aWidth);

                        if ($(".animation div").index(this) == 24) {
                            delay = 700;
                        }
                        $(this).css({ "height": gap.height, "width": gap.width, "opacity": 0.01 }).delay(delay).animate({ "opacity": 1 }, self.options.animationSpeed, function () {
                            if (self.aniDiv.find(" div").index(this) == 24) {
                                self.finishAnimating(data);
                            }
                        });
                    });
                    break;
                case 6:
                    //full frame slide
                    self.aniDiv.height(self.image.height()).hide().css({ 'background': 'url(' + data.source + ') top left no-repeat' });
                    self.aniDiv.children('div').hide();
                    self.aniDiv.css({ width: 0 }).show().animate({ width: self.image.width() }, self.options.animationSpeed, function () {
                        self.finishAnimating(data);
                        self.aniDiv.css({ 'background': 'transparent' });
                    });
                    break;
                case 7:
                    //side in slide
                    self.wrap.css({ overflow: 'hidden' });
                    /* My code */
                    self.imgResize(self.aniDiv, self.aniImg, 0, 0);
                    self.aniImg.hide().attr('src', data.source);
                    /* End My code */
                    /* ORIGINAL CODE
					self.aniImg.height(self.image.height()).hide().attr('src', data.source);
					*/
                    self.aniDiv.hide();
                    /* My code */
                    var intLeft = self.wrap.outerWidth();
                    if (self.intDifference < 0) {
                        intLeft = -intLeft;
                    }
                    self.image.css({ position: 'relative' }).animate({ left: -intLeft + "px" }, self.options.animationSpeed); // EDITED ADDED OVERFLOW
                    self.aniImg.show();
                    self.aniWrap.css({ left: intLeft }).show().animate({ left: "0px" }, self.options.animationSpeed, function () {
                        self.finishAnimating(data);
                    });
                    /* End My code */
                    /* ORIGINAL CODE
					self.image.css({ position: 'relative' }).animate({ left: "-" + self.wrap.outerWidth() + "px" }, self.options.animationSpeed);
					self.aniImg.show();
					self.aniWrap.css({ left: self.wrap.outerWidth() }).show().animate({ left: "0px" }, self.options.animationSpeed, function ()
					{
						self.finishAnimating(data);
					});
					*/
                    break;
            }
        },//end doAnimation
        /**
		* Executes after the animation finishes, resets the animation divs
		*
		* @method finishAnimating
		* @return undefined
		*/
        finishAnimating: function (data) {
            this.animating = false;
            this.image.attr('src', data.source);
            this.image.attr('alt', data.imageAlt);
            this.image.attr('title', data.imageTitle);
            this.image.css({ left: "0" });
            this.image.show();
            var self = this;
            $('<img />').on('load', function () {
                /* My Code */
                // Needed to render smaller images and center them in the middle of the gallery item display
                if (self.dfdLoaded.state() === "resolved") {
                    self.imgResize(self.aniDiv, self.image, self.image.width(), self.image.height());
                }
                /* End My Code */
                self.aniImg.fadeOut('fast');
                self.aniDiv.fadeOut('fast');
            }).attr('src', data.source);

            var cur = data.index + 1;
            var total = this.thumbs.length;
            this.counter.html(cur + "/" + total);
            /* My Code */
            if (this.dfdLoaded.state() === "resolved") {
                this.imgResize(this.aniDiv, this.image, data.width, data.height);
            }
            /* End My Code */
            if (data.clickThrough !== "") {
                if (this.anchor === null) {
                    this.anchor = this.image.wrap("<a>").parent();
                }
                this.anchor.attr('href', data.clickThrough);
                this.anchor.attr('title', data.clickThroughTitle);
                this.anchor.attr('target', data.clickThroughTarget);
            } else {
                if (this.image.parent('a').length > 0) { this.image.unwrap(); }
                this.anchor = null;
            }
            if (this.options.showCaption && !!data.caption && data.caption.length > 0) {
                /* My Code */
                this.captionText.html(data.caption);
                this.caption.fadeTo('slow', 1);
                /* End My Code */
                /* ORIGINAL CODE
				this.caption.html(data.caption).fadeTo('slow', 1);
				*/
            }
                /* My Code */
            else {
                this.captionText.empty();
                this.caption.hide();
            }
            /* End My Code */
            if (this.options.autoPlay && total > 1) {
                this.timeOut = setTimeout((function (self) {
                    return function () { self.nextClick(); };
                })(this), this.options.speed, this.timeOut);
            }

            if (typeof (this.options.animationFinished) == 'function') {
                this.options.animationFinished(this);
            }
        },//end finishedAnimating
        /**
		* Makes sure the rows and columns are the full width/heigh tof the image
		*
		* @method gapper
		* @return object {height:int,width:int}
		*/
        gapper: function (ele, aHeight, aWidth) {
            var gap;
            if (ele.attr('row') == 4) {
                //last row, check the gap and fix it!
                gap = (this.aniDiv.height() - (aHeight * 5)) + aHeight;
                aHeight = gap;
            }
            if (ele.attr('col') == 4) {
                //last row, check the gap and fix it!
                gap = (this.aniDiv.width() - (aWidth * 5)) + aWidth;
                aWidth = gap;
            }
            return { height: aHeight, width: aWidth };
        },
        nextClick: function (e) {
            var how = "natural";
            var self = null;
            try {
                self = e.data.self;
                if (typeof (e.data.self.options.next) == 'function') {
                    e.data.self.options.next(this);
                }
            } catch (err) {
                self = this;
                how = "auto";
            }
            /* My Code */
            self.intDifference = 1;
            var reset = false;
            /* End My Code */
            var next = self.active.parents('li:first').next().find('img');
            if (next.length === 0) {
                next = self.list.find('img').eq(0);
                /* My Code */
                reset = true;
                if (self.usePagination) {
                    self.manualControls
						.find("li")
						.filter(".currentItem")
						.removeClass("currentItem")
						.end()
						.first()
						.addClass("currentItem");
                }
                /* End My Code */
            }
            /* My Code */
            if (self.options.shuffleItems) {
                next = self.list.find("img")
					.eq(Math.floor(Math.random() * self.list.find("img").length));
            }
            if (self.usePagination &&
				how == "auto" && reset !== true) {
                self.manualControls
					.find("li")
					.filter(".currentItem")
					.removeClass("currentItem")
					.end()
					.eq(self.list.find("img").index(self.active) + 1)
					.addClass("currentItem");
            }
            /* End My Code */
            next.trigger('click', { how: how });
            return next;
        },
        prevClick: function (e) {
            if (typeof (e.data.self.options.previous) == 'function') {
                e.data.self.options.previous(this);
            }
            var self = e.data.self;
            /* My Code */
            self.intDifference = -1;
            /* End My Code */
            var prev = self.active.parents('li:first').prev().find('img');
            if (prev.length === 0) { prev = self.list.find('img:last'); }
            prev.trigger('click');
            return prev;
        },
        playClick: function (e) {
            var self = e.data.self;
            self.options.autoPlay = !self.options.autoPlay;
            self.imgPlay.toggleClass('play').toggleClass('pause');
            if (self.options.autoPlay) { self.nextClick(); }
            else
            {
                //clear timeout to prevent multiple clicks being sent
                clearTimeout(self.timeOut);
            }
            return (self.options.autoPlay) ? "playing" : "paused";
        },
        Next: function () {
            var e = { data: { self: this } };
            return this.nextClick(e);
        },
        Prev: function () {
            var e = { data: { self: this } };
            return this.prevClick(e);
        },
        GoTo: function (v) {
            var e = { data: { self: this } };
            this.intDifference = this.active.data("index") < v ? 1 : -1;
            var gotoImg = this.list.find('img').eq(v);
            if (gotoImg.length > 0) {
                gotoImg.trigger('click');
                return gotoImg;
            }
            else {
                throw "Image not found. Images are 0 indexed.";
            }
        },
        Play: function () {
            if (this.options.autoPlay) { return "playing"; }
            var e = { data: { self: this } };
            return this.playClick(e);
        },
        Pause: function () {
            if (!this.options.autoPlay) { return "paused"; }
            var e = { data: { self: this } };
            return this.playClick(e);
        },
        /**
		* Re processes the list of images stored in this.list
		* Useful when dynamically adding images to PikaChoose
		*
		* @method finishAnimating
		* @return undefined
		*/
        updateThumbs: function () {
            var self = this;
            this.thumbs = this.list.find('img');
            this.thumbs.each(function () {
                self.createThumb($(this), self);
            });
        }
        /* My Code */
		, imgResize: function (img, imgOut, intWidth, intHeight) {
		    img = $(img);
		    imgOut = $(imgOut);

		    var w = img.width();
		    var h = img.height();
		    if (!!intHeight && !!intWidth) {
		        w = intWidth;
		        h = intHeight;
		    }

		    if (w === 0) { w = img.attr('width'); }
		    if (h === 0) { h = img.attr('height'); }
		    if (w === 0 || !w) { w = intWidth; }
		    if (h === 0 || !h) { h = intHeight; }

		    //grab a ratio for image to user defined settings
		    var rw = 1;
		    var rh = 1;
		    var left;
		    var top;
		    if (w) {
		        rw = this.options.mainItemWidth / w;
		    }
		    if (h) {
		        rh = this.options.mainItemHeight / h;
		    }

		    imgOut
				.width(w)
				.height(h)
				.css({
				    left: '',
				    top: '',
				    position: 'relative'
				});

		    if (rw > 1 && rh > 1) {
		        left = (this.options.mainItemWidth - w) / 2;
		        left = Math.round(left);

		        top = (this.options.mainItemHeight - h) / 2;
		        top = Math.round(top);
		        imgOut.css({
		            position: 'relative',
		            left: left + 'px',
		            top: top + 'px'
		        });
		    }
		    else {
		        //determine which has the smallest ratio (thus needing
		        //to be the side we use to scale so our whole thumb is filled)
		        var ratio;
		        if (rw > rh) {
		            //we'll use ratio later to scale and not distort
		            ratio = rh;
		            left = ((w * ratio - this.options.mainItemWidth) / 2) * -1;
		            left = Math.round(left);
		            imgOut.css({ left: left + 'px' });
		        }
		        else {
		            ratio = rw;
		            top = ((h * ratio - this.options.mainItemHeight) / 2) * -1;
		            top = Math.round(top);
		            imgOut.css({ top: top + 'px' });
		        }
		        //use those ratios to calculate scale
		        var width = Math.round(w * ratio);
		        var height = Math.round(h * ratio);
		        imgOut
					.width(width)
					.height(height)
					.css("position", 'relative');
		    }
		},
        isOldIE: function () {
            var ua = navigator.userAgent.toLowerCase(),
				match = /(msie)\s?([0-9]+)\.[0-9a-z]+\s*;/.exec(ua) || [],
				browser = match[1] || "9";

            return parseInt(browser, 10) < 9;
        },
        getDirection: function (self, c) {
            var difference = self.active.data("index") - c;
            return difference !== 0 ? (difference < 0 ? -1 : 1) : 0;
        }
    }); //end extend
})(jQuery);